@@ -34,6 +34,7 @@ describe('esprima.Syntax', function () {
             ArrowFunctionExpression: 'ArrowFunctionExpression',
             AssignmentExpression: 'AssignmentExpression',
             AssignmentPattern: 'AssignmentPattern',
+            AwaitExpression: 'AwaitExpression',
             BinaryExpression: 'BinaryExpression',
             BlockStatement: 'BlockStatement',
             BreakStatement: 'BreakStatement',
@@ -275,6 +276,365 @@ describe('esprima.parse', function () {
     });
 });
 
+describe('esprima.parseModule', function () {
+
+    it('should not accept zero parameter', function () {
+        assert.throws(function () {
+            esprima.parseModule();
+        });
+    });
+
+    it('should accept one string parameter', function () {
+        assert.doesNotThrow(function () {
+            esprima.parseModule('function f(){}');
+        });
+    });
+
+    it('should accept a String object', function () {
+        assert.doesNotThrow(function () {
+            esprima.parseModule(new String('some.property = value'));
+        });
+    });
+
+    it('should accept two parameters (source and parsing options)', function () {
+        var options = { range: false };
+        assert.doesNotThrow(function () {
+            esprima.parseModule('var x', options);
+        });
+    });
+
+    it('should accept three parameters (source, options, and delegate)', function () {
+        var options = { range: false };
+        assert.doesNotThrow(function () {
+            esprima.parseModule('var x', options, function f(entry) {
+                return entry;
+            });
+        });
+    });
+
+    it('should exclude location information by default', function () {
+        var ast = esprima.parseModule('answer = 42');
+        assert.ifError(ast.range);
+        assert.ifError(ast.loc);
+    });
+
+    it('should exclude comments by default', function () {
+        var ast = esprima.parseModule('42 // answer');
+        assert.ifError(ast.comments);
+    });
+
+    it('should exclude list of tokens by default', function () {
+        var ast = esprima.parseModule('3.14159');
+        assert.ifError(ast.tokens);
+    });
+
+    it('should include index-based location for the nodes when specified', function () {
+        var ast = esprima.parseModule('answer = 42', { range: true });
+        assert.deepEqual(ast.range, [0, 11]);
+        assert.ifError(ast.loc);
+    });
+
+    it('should include line and column-based location for the nodes when specified', function () {
+        var ast = esprima.parseModule('answer = 42', { loc: true });
+        assert.deepEqual(ast.loc.start, { line: 1, column: 0 });
+        assert.deepEqual(ast.loc.end, { line: 1, column: 11 });
+        assert.ifError(ast.loc.source);
+        assert.ifError(ast.range);
+    });
+
+    it('should include source information for the nodes when specified', function () {
+        var ast = esprima.parseModule('answer = 42', { loc: true, source: 'example.js' });
+        assert.deepEqual(ast.loc.source, 'example.js');
+        assert.ifError(ast.range);
+    });
+
+    it('should include the list of comments when specified', function () {
+        var ast = esprima.parseModule('42 // answer', { comment: true });
+        assert.deepEqual(ast.comments.length, 1);
+        assert.deepEqual(ast.comments[0], { type: 'Line', value: ' answer' });
+    });
+
+    it('should include the list of comments with index-based location when specified', function () {
+        var ast = esprima.parseModule('42 // answer', { comment: true, range: true });
+        assert.deepEqual(ast.comments.length, 1);
+        assert.deepEqual(ast.comments[0], { type: 'Line', value: ' answer', range: [3, 12] });
+    });
+
+    it('should include the list of comments with line and column-based location when specified', function () {
+        var ast = esprima.parseModule('42 // answer', { comment: true, loc: true });
+        assert.deepEqual(ast.comments.length, 1);
+        assert.deepEqual(ast.comments[0].type, 'Line');
+        assert.deepEqual(ast.comments[0].value, ' answer');
+        assert.deepEqual(ast.comments[0].loc.start, { line: 1, column: 3 });
+        assert.deepEqual(ast.comments[0].loc.end, { line: 1, column: 12 });
+        assert.ifError(ast.comments[0].range);
+    });
+
+    it('should be able to attach comments', function () {
+        var ast, statement, expression, comments;
+
+        ast = esprima.parseModule('/* universe */ 42', { attachComment: true });
+        statement = ast.body[0];
+        expression = statement.expression;
+        comments = statement.leadingComments;
+
+        assert.deepEqual(expression, { type: 'Literal', value: 42, raw: '42' });
+        assert.deepEqual(statement.leadingComments, [{ type: 'Block', value: ' universe ', range: [0, 14] }]);
+    });
+
+    it('should not implicity collect comments when comment attachment is specified', function () {
+        var ast = esprima.parseModule('/* universe */ 42', { attachComment: true });
+        assert.equal(ast.comments, undefined);
+    });
+
+    it('should include the list of tokens when specified', function () {
+        var ast = esprima.parseModule('x = 1', { tokens: true });
+        assert.deepEqual(ast.tokens.length, 3);
+        assert.deepEqual(ast.tokens[0], { type: 'Identifier', value: 'x' });
+        assert.deepEqual(ast.tokens[1], { type: 'Punctuator', value: '=' });
+        assert.deepEqual(ast.tokens[2], { type: 'Numeric', value: '1' });
+    });
+
+    it('should include the list of tokens with index-based location when specified', function () {
+        var ast = esprima.parseModule('y = 2', { tokens: true, range: true });
+        assert.deepEqual(ast.tokens[0], { type: 'Identifier', value: 'y', range: [0, 1] });
+        assert.deepEqual(ast.tokens[1], { type: 'Punctuator', value: '=', range: [2, 3] });
+        assert.deepEqual(ast.tokens[2], { type: 'Numeric', value: '2', range: [4, 5] });
+    });
+
+    it('should include the list of tokens with line and column-based location when specified', function () {
+        var ast = esprima.parseModule('z = 3', { tokens: true, loc: true });
+        assert.deepEqual(ast.tokens.length, 3);
+        assert.deepEqual(ast.tokens[0].type, 'Identifier');
+        assert.deepEqual(ast.tokens[0].value, 'z');
+        assert.deepEqual(ast.tokens[0].loc.start, { line: 1, column: 0 });
+        assert.deepEqual(ast.tokens[0].loc.end, { line: 1, column: 1 });
+        assert.ifError(ast.tokens[0].range);
+    });
+
+    it('should not understand JSX syntax by default', function () {
+        assert.throws(function () {
+            esprima.parseModule('<head/>');
+        });
+    });
+
+    it('should not understand JSX syntax if it is explicitly not to be supported', function () {
+        assert.throws(function () {
+            esprima.parseModule('<b/>', { jsx: false });
+        });
+    });
+
+    it('should understand JSX syntax if specified', function () {
+        var ast, statement, expression;
+        ast = esprima.parseModule('<title/>', { jsx: true });
+        statement = ast.body[0];
+        expression = statement.expression;
+
+        assert.deepEqual(expression.type, 'JSXElement');
+        assert.deepEqual(expression.openingElement.type, 'JSXOpeningElement');
+        assert.deepEqual(expression.openingElement.name, { type: 'JSXIdentifier', name: 'title' });
+        assert.deepEqual(expression.closingElement, null);
+    });
+
+    it('should never produce shallow copied nodes', function () {
+        var ast, pattern, expr;
+        ast = esprima.parseModule('let {a, b} = {x, y, z}');
+        pattern = ast.body[0].declarations[0].id;
+        expr = ast.body[0].declarations[0].init;
+        pattern.properties[0].key.name = 'foo';
+        expr.properties[0].key.name = 'bar';
+
+        assert.deepEqual(pattern.properties[0].value, { type: 'Identifier', name: 'a' });
+        assert.deepEqual(pattern.properties[1].value, { type: 'Identifier', name: 'b' });
+        assert.deepEqual(expr.properties[0].value, { type: 'Identifier', name: 'x' });
+        assert.deepEqual(expr.properties[1].value, { type: 'Identifier', name: 'y' });
+        assert.deepEqual(expr.properties[2].value, { type: 'Identifier', name: 'z' });
+    });
+
+    it('should perform strict mode parsing', function () {
+        assert.throws(function () {
+            esprima.parseModule('with (x) {}');
+        });
+    });
+
+});
+
+describe('esprima.parseScript', function () {
+
+    it('should not accept zero parameter', function () {
+        assert.throws(function () {
+            esprima.parseScript();
+        });
+    });
+
+    it('should accept one string parameter', function () {
+        assert.doesNotThrow(function () {
+            esprima.parseScript('function f(){}');
+        });
+    });
+
+    it('should accept a String object', function () {
+        assert.doesNotThrow(function () {
+            esprima.parseScript(new String('some.property = value'));
+        });
+    });
+
+    it('should accept two parameters (source and parsing options)', function () {
+        var options = { range: false };
+        assert.doesNotThrow(function () {
+            esprima.parseScript('var x', options);
+        });
+    });
+
+    it('should accept three parameters (source, options, and delegate)', function () {
+        var options = { range: false };
+        assert.doesNotThrow(function () {
+            esprima.parseScript('var x', options, function f(entry) {
+                return entry;
+            });
+        });
+    });
+
+    it('should exclude location information by default', function () {
+        var ast = esprima.parseScript('answer = 42');
+        assert.ifError(ast.range);
+        assert.ifError(ast.loc);
+    });
+
+    it('should exclude comments by default', function () {
+        var ast = esprima.parseScript('42 // answer');
+        assert.ifError(ast.comments);
+    });
+
+    it('should exclude list of tokens by default', function () {
+        var ast = esprima.parseScript('3.14159');
+        assert.ifError(ast.tokens);
+    });
+
+    it('should include index-based location for the nodes when specified', function () {
+        var ast = esprima.parseScript('answer = 42', { range: true });
+        assert.deepEqual(ast.range, [0, 11]);
+        assert.ifError(ast.loc);
+    });
+
+    it('should include line and column-based location for the nodes when specified', function () {
+        var ast = esprima.parseScript('answer = 42', { loc: true });
+        assert.deepEqual(ast.loc.start, { line: 1, column: 0 });
+        assert.deepEqual(ast.loc.end, { line: 1, column: 11 });
+        assert.ifError(ast.loc.source);
+        assert.ifError(ast.range);
+    });
+
+    it('should include source information for the nodes when specified', function () {
+        var ast = esprima.parseScript('answer = 42', { loc: true, source: 'example.js' });
+        assert.deepEqual(ast.loc.source, 'example.js');
+        assert.ifError(ast.range);
+    });
+
+    it('should include the list of comments when specified', function () {
+        var ast = esprima.parseScript('42 // answer', { comment: true });
+        assert.deepEqual(ast.comments.length, 1);
+        assert.deepEqual(ast.comments[0], { type: 'Line', value: ' answer' });
+    });
+
+    it('should include the list of comments with index-based location when specified', function () {
+        var ast = esprima.parseScript('42 // answer', { comment: true, range: true });
+        assert.deepEqual(ast.comments.length, 1);
+        assert.deepEqual(ast.comments[0], { type: 'Line', value: ' answer', range: [3, 12] });
+    });
+
+    it('should include the list of comments with line and column-based location when specified', function () {
+        var ast = esprima.parseScript('42 // answer', { comment: true, loc: true });
+        assert.deepEqual(ast.comments.length, 1);
+        assert.deepEqual(ast.comments[0].type, 'Line');
+        assert.deepEqual(ast.comments[0].value, ' answer');
+        assert.deepEqual(ast.comments[0].loc.start, { line: 1, column: 3 });
+        assert.deepEqual(ast.comments[0].loc.end, { line: 1, column: 12 });
+        assert.ifError(ast.comments[0].range);
+    });
+
+    it('should be able to attach comments', function () {
+        var ast, statement, expression, comments;
+
+        ast = esprima.parseScript('/* universe */ 42', { attachComment: true });
+        statement = ast.body[0];
+        expression = statement.expression;
+        comments = statement.leadingComments;
+
+        assert.deepEqual(expression, { type: 'Literal', value: 42, raw: '42' });
+        assert.deepEqual(statement.leadingComments, [{ type: 'Block', value: ' universe ', range: [0, 14] }]);
+    });
+
+    it('should not implicity collect comments when comment attachment is specified', function () {
+        var ast = esprima.parseScript('/* universe */ 42', { attachComment: true });
+        assert.equal(ast.comments, undefined);
+    });
+
+    it('should include the list of tokens when specified', function () {
+        var ast = esprima.parseScript('x = 1', { tokens: true });
+        assert.deepEqual(ast.tokens.length, 3);
+        assert.deepEqual(ast.tokens[0], { type: 'Identifier', value: 'x' });
+        assert.deepEqual(ast.tokens[1], { type: 'Punctuator', value: '=' });
+        assert.deepEqual(ast.tokens[2], { type: 'Numeric', value: '1' });
+    });
+
+    it('should include the list of tokens with index-based location when specified', function () {
+        var ast = esprima.parseScript('y = 2', { tokens: true, range: true });
+        assert.deepEqual(ast.tokens[0], { type: 'Identifier', value: 'y', range: [0, 1] });
+        assert.deepEqual(ast.tokens[1], { type: 'Punctuator', value: '=', range: [2, 3] });
+        assert.deepEqual(ast.tokens[2], { type: 'Numeric', value: '2', range: [4, 5] });
+    });
+
+    it('should include the list of tokens with line and column-based location when specified', function () {
+        var ast = esprima.parseScript('z = 3', { tokens: true, loc: true });
+        assert.deepEqual(ast.tokens.length, 3);
+        assert.deepEqual(ast.tokens[0].type, 'Identifier');
+        assert.deepEqual(ast.tokens[0].value, 'z');
+        assert.deepEqual(ast.tokens[0].loc.start, { line: 1, column: 0 });
+        assert.deepEqual(ast.tokens[0].loc.end, { line: 1, column: 1 });
+        assert.ifError(ast.tokens[0].range);
+    });
+
+    it('should not understand JSX syntax by default', function () {
+        assert.throws(function () {
+            esprima.parseScript('<head/>');
+        });
+    });
+
+    it('should not understand JSX syntax if it is explicitly not to be supported', function () {
+        assert.throws(function () {
+            esprima.parseScript('<b/>', { jsx: false });
+        });
+    });
+
+    it('should understand JSX syntax if specified', function () {
+        var ast, statement, expression;
+        ast = esprima.parseScript('<title/>', { jsx: true });
+        statement = ast.body[0];
+        expression = statement.expression;
+
+        assert.deepEqual(expression.type, 'JSXElement');
+        assert.deepEqual(expression.openingElement.type, 'JSXOpeningElement');
+        assert.deepEqual(expression.openingElement.name, { type: 'JSXIdentifier', name: 'title' });
+        assert.deepEqual(expression.closingElement, null);
+    });
+
+    it('should never produce shallow copied nodes', function () {
+        var ast, pattern, expr;
+        ast = esprima.parseScript('let {a, b} = {x, y, z}');
+        pattern = ast.body[0].declarations[0].id;
+        expr = ast.body[0].declarations[0].init;
+        pattern.properties[0].key.name = 'foo';
+        expr.properties[0].key.name = 'bar';
+
+        assert.deepEqual(pattern.properties[0].value, { type: 'Identifier', name: 'a' });
+        assert.deepEqual(pattern.properties[1].value, { type: 'Identifier', name: 'b' });
+        assert.deepEqual(expr.properties[0].value, { type: 'Identifier', name: 'x' });
+        assert.deepEqual(expr.properties[1].value, { type: 'Identifier', name: 'y' });
+        assert.deepEqual(expr.properties[2].value, { type: 'Identifier', name: 'z' });
+    });
+});
+
 describe('esprima.parse delegate', function () {
 
     it('should receive all the nodes', function () {
@@ -343,6 +703,24 @@ describe('esprima.parse delegate', function () {
         assert.deepEqual(list[5], 'ExpressionStatement');
     });
 
+    it('should be affected by async arrow function cover grammar', function () {
+        var list = [];
+        esprima.parse('async (a) => 1', {}, function (n) {
+            list.push(n.type)
+        });
+
+        // async (a) will be a CallExpression first before being reinterpreted as
+        // the formal parameter list for an (async) ArrowFunctionExpression
+
+        assert.deepEqual(list.length, 7);
+        assert.deepEqual(list[0], 'Identifier');               // async
+        assert.deepEqual(list[1], 'Identifier');               // a
+        assert.deepEqual(list[2], 'CallExpression');           // async (a)
+        assert.deepEqual(list[3], 'Literal');                  // 1
+        assert.deepEqual(list[4], 'ArrowFunctionExpression');  // async (a) => 1
+        assert.deepEqual(list[5], 'ExpressionStatement');
+    });
+
     it('should receive metadata of each node', function () {
         var starts = [], ends = [];
         esprima.parse('x = y + z', { range: false }, function (node, metadata) {
